@@ -13,18 +13,21 @@ $notes :ref:`Low latency streaming <cli/plugins/twitch:Low latency streaming>` i
 $notes Acquires a :ref:`client-integrity token <cli/plugins/twitch:Client-integrity token>` on streaming access token failure.
 """
 
+from __future__ import annotations
+
 import argparse
 import logging
 import math
 import re
 import sys
 from collections import deque
+from collections.abc import Mapping
 from contextlib import suppress
 from dataclasses import dataclass, replace as dataclass_replace
 from datetime import datetime, timedelta
 from json import dumps as json_dumps
 from random import random
-from typing import ClassVar, Deque, List, Mapping, Optional, Tuple, Type
+from typing import ClassVar
 from urllib.parse import urlparse
 
 from requests.exceptions import HTTPError
@@ -66,12 +69,12 @@ class TwitchHLSSegment(HLSSegment):
 class TwitchM3U8(M3U8[TwitchHLSSegment, HLSPlaylist]):
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
-        self.dateranges_ads: List[DateRange] = []
+        self.dateranges_ads: list[DateRange] = []
 
 
 class TwitchM3U8Parser(M3U8Parser[TwitchM3U8, TwitchHLSSegment, HLSPlaylist]):
-    __m3u8__: ClassVar[Type[TwitchM3U8]] = TwitchM3U8
-    __segment__: ClassVar[Type[TwitchHLSSegment]] = TwitchHLSSegment
+    __m3u8__: ClassVar[type[TwitchM3U8]] = TwitchM3U8
+    __segment__: ClassVar[type[TwitchHLSSegment]] = TwitchHLSSegment
 
     @parse_tag("EXT-X-TWITCH-LIVE-SEQUENCE")
     def parse_ext_x_twitch_live_sequence(self, value):
@@ -135,16 +138,16 @@ class TwitchM3U8Parser(M3U8Parser[TwitchM3U8, TwitchHLSSegment, HLSPlaylist]):
             and not segment.ad
             and self.m3u8.segments
             and not self.m3u8.segments[-1].ad
-        ):
+        ):  # fmt: skip
             segment.discontinuity = False
 
         return segment
 
-    def _is_segment_ad(self, date: Optional[datetime], title: Optional[str] = None) -> bool:
+    def _is_segment_ad(self, date: datetime | None, title: str | None = None) -> bool:
         return (
             title is not None and "Amazon" in title
             or any(self.m3u8.is_date_in_daterange(date, daterange) for daterange in self.m3u8.dateranges_ads)
-        )
+        )  # fmt: skip
 
     @staticmethod
     def _is_daterange_ad(daterange: DateRange) -> bool:
@@ -156,13 +159,13 @@ class TwitchM3U8Parser(M3U8Parser[TwitchM3U8, TwitchHLSSegment, HLSPlaylist]):
 
 
 class TwitchHLSStreamWorker(HLSStreamWorker):
-    reader: "TwitchHLSStreamReader"
-    writer: "TwitchHLSStreamWriter"
-    stream: "TwitchHLSStream"
+    reader: TwitchHLSStreamReader
+    writer: TwitchHLSStreamWriter
+    stream: TwitchHLSStream
 
     def __init__(self, reader, *args, **kwargs) -> None:
         self.had_content: bool = False
-        self.logged_ads: Deque[str] = deque(maxlen=10)
+        self.logged_ads: deque[str] = deque(maxlen=10)
         super().__init__(reader, *args, **kwargs)
 
     def _playlist_reload_time(self, playlist: TwitchM3U8):  # type: ignore[override]
@@ -198,10 +201,10 @@ class TwitchHLSStreamWorker(HLSStreamWorker):
             if not daterange_ads.duration:  # pragma: no cover
                 continue
 
-            ads_id: Optional[str] = (
+            ads_id: str | None = (
                 daterange_ads.x.get("X-TV-TWITCH-AD-COMMERCIAL-ID")
                 or daterange_ads.x.get("X-TV-TWITCH-AD-ROLL-TYPE")
-            )
+            )  # fmt: skip
             if not ads_id or ads_id in self.logged_ads:
                 continue
             self.logged_ads.append(ads_id)
@@ -218,8 +221,8 @@ class TwitchHLSStreamWorker(HLSStreamWorker):
 
 
 class TwitchHLSStreamWriter(HLSStreamWriter):
-    reader: "TwitchHLSStreamReader"
-    stream: "TwitchHLSStream"
+    reader: TwitchHLSStreamReader
+    stream: TwitchHLSStream
 
     def should_filter_segment(self, segment: TwitchHLSSegment) -> bool:  # type: ignore[override]
         return self.stream.disable_ads and segment.ad
@@ -229,11 +232,11 @@ class TwitchHLSStreamReader(HLSStreamReader):
     __worker__ = TwitchHLSStreamWorker
     __writer__ = TwitchHLSStreamWriter
 
-    worker: "TwitchHLSStreamWorker"
-    writer: "TwitchHLSStreamWriter"
-    stream: "TwitchHLSStream"
+    worker: TwitchHLSStreamWorker
+    writer: TwitchHLSStreamWriter
+    stream: TwitchHLSStream
 
-    def __init__(self, stream: "TwitchHLSStream"):
+    def __init__(self, stream: TwitchHLSStream):
         if stream.disable_ads:
             log.info("Will skip ad segments")
         if stream.low_latency:
@@ -307,16 +310,21 @@ class TwitchAPI:
         self.access_token_params = dict(access_token_param or [])
         self.access_token_params.setdefault("playerType", "embed")
 
-    def call(self, data, schema=None, **kwargs):
-        res = self.session.http.post(
+    def call(self, data, /, *, headers=None, schema, **kwargs):
+        return self.session.http.post(
             "https://gql.twitch.tv/gql",
             json=data,
-            headers={**self.headers, **kwargs.pop("headers", {})},
+            headers={
+                **self.headers,
+                **(headers or {}),
+            },
+            schema=validate.Schema(
+                validate.parse_json(),
+                schema,
+            ),
             **kwargs,
         )
 
-        return self.session.http.json(res, schema=schema)
-
     @staticmethod
     def _gql_persisted_query(operationname, sha256hash, **variables):
         return {
@@ -332,13 +340,20 @@ class TwitchAPI:
 
     @staticmethod
     def parse_token(tokenstr):
-        return parse_json(tokenstr, schema=validate.Schema(
-            {"chansub": {"restricted_bitrates": validate.all(
-                [str],
-                validate.filter(lambda n: not re.match(r"(.+_)?archives|live|chunked", n)),
-            )}},
-            validate.get(("chansub", "restricted_bitrates")),
-        ))
+        return parse_json(
+            tokenstr,
+            schema=validate.Schema(
+                {
+                    "chansub": {
+                        "restricted_bitrates": validate.all(
+                            [str],
+                            validate.filter(lambda n: not re.match(r"(.+_)?archives|live|chunked", n)),
+                        ),
+                    },
+                },
+                validate.get(("chansub", "restricted_bitrates")),
+            ),
+        )
 
     # GraphQL API calls
 
@@ -350,25 +365,32 @@ class TwitchAPI:
             videoID=video_id,
         )
 
-        return self.call(query, schema=validate.Schema(
-            {"data": {"video": {
-                "id": str,
-                "owner": {
-                    "displayName": str,
+        return self.call(
+            query,
+            schema=validate.all(
+                {
+                    "data": {
+                        "video": {
+                            "id": str,
+                            "owner": {
+                                "displayName": str,
+                            },
+                            "title": str,
+                            "game": {
+                                "displayName": str,
+                            },
+                        },
+                    },
                 },
-                "title": str,
-                "game": {
-                    "displayName": str,
-                },
-            }}},
-            validate.get(("data", "video")),
-            validate.union_get(
-                "id",
-                ("owner", "displayName"),
-                ("game", "displayName"),
-                "title",
+                validate.get(("data", "video")),
+                validate.union_get(
+                    "id",
+                    ("owner", "displayName"),
+                    ("game", "displayName"),
+                    "title",
+                ),
             ),
-        ))
+        )
 
     def metadata_channel(self, channel):
         queries = [
@@ -385,34 +407,45 @@ class TwitchAPI:
             ),
         ]
 
-        return self.call(queries, schema=validate.Schema(
-            [
-                validate.all(
-                    {"data": {"userOrError": {
-                        "displayName": str,
-                    }}},
-                ),
-                validate.all(
-                    {"data": {"user": {
-                        "lastBroadcast": {
-                            "title": str,
-                        },
-                        "stream": {
-                            "id": str,
-                            "game": {
-                                "name": str,
+        return self.call(
+            queries,
+            schema=validate.all(
+                validate.list(
+                    validate.all(
+                        {
+                            "data": {
+                                "userOrError": {
+                                    "displayName": str,
+                                },
                             },
                         },
-                    }}},
+                    ),
+                    validate.all(
+                        {
+                            "data": {
+                                "user": {
+                                    "lastBroadcast": {
+                                        "title": str,
+                                    },
+                                    "stream": {
+                                        "id": str,
+                                        "game": {
+                                            "name": str,
+                                        },
+                                    },
+                                },
+                            },
+                        },
+                    ),
+                ),
+                validate.union_get(
+                    (1, "data", "user", "stream", "id"),
+                    (0, "data", "userOrError", "displayName"),
+                    (1, "data", "user", "stream", "game", "name"),
+                    (1, "data", "user", "lastBroadcast", "title"),
                 ),
-            ],
-            validate.union_get(
-                (1, "data", "user", "stream", "id"),
-                (0, "data", "userOrError", "displayName"),
-                (1, "data", "user", "stream", "game", "name"),
-                (1, "data", "user", "lastBroadcast", "title"),
             ),
-        ))
+        )
 
     def metadata_clips(self, clipname):
         queries = [
@@ -428,30 +461,37 @@ class TwitchAPI:
             ),
         ]
 
-        return self.call(queries, schema=validate.Schema(
-            [
-                validate.all(
-                    {"data": {"clip": {
-                        "id": str,
-                        "broadcaster": {"displayName": str},
-                        "game": {"name": str},
-                    }}},
-                    validate.get(("data", "clip")),
+        return self.call(
+            queries,
+            schema=validate.all(
+                validate.list(
+                    validate.all(
+                        {
+                            "data": {
+                                "clip": {
+                                    "id": str,
+                                    "broadcaster": {"displayName": str},
+                                    "game": {"name": str},
+                                },
+                            },
+                        },
+                        validate.get(("data", "clip")),
+                    ),
+                    validate.all(
+                        {"data": {"clip": {"title": str}}},
+                        validate.get(("data", "clip")),
+                    ),
                 ),
-                validate.all(
-                    {"data": {"clip": {"title": str}}},
-                    validate.get(("data", "clip")),
+                validate.union_get(
+                    (0, "id"),
+                    (0, "broadcaster", "displayName"),
+                    (0, "game", "name"),
+                    (1, "title"),
                 ),
-            ],
-            validate.union_get(
-                (0, "id"),
-                (0, "broadcaster", "displayName"),
-                (0, "game", "name"),
-                (1, "title"),
             ),
-        ))
+        )
 
-    def access_token(self, is_live, channel_or_vod, client_integrity: Optional[Tuple[str, str]] = None):
+    def access_token(self, is_live, channel_or_vod, client_integrity: tuple[str, str] | None = None):
         query = self._gql_persisted_query(
             "PlaybackAccessToken",
             "0828119ded1c13477966434e15800ff57ddacf13ba1911c129dc2200705b0712",
@@ -473,8 +513,11 @@ class TwitchAPI:
         if client_integrity:
             headers["Device-Id"], headers["Client-Integrity"] = client_integrity
 
-        return self.call(query, acceptable_status=(200, 400, 401, 403), headers=headers, schema=validate.Schema(
-            validate.any(
+        return self.call(
+            query,
+            acceptable_status=(200, 400, 401, 403),
+            headers=headers,
+            schema=validate.any(
                 validate.all(
                     {"errors": [{"message": str}]},
                     validate.get(("errors", 0, "message")),
@@ -502,7 +545,7 @@ class TwitchAPI:
                     validate.transform(lambda data: ("token", *data) if data is not None else ("token", None, None)),
                 ),
             ),
-        ))
+        )
 
     def clips(self, clipname):
         query = self._gql_persisted_query(
@@ -511,31 +554,42 @@ class TwitchAPI:
             slug=clipname,
         )
 
-        return self.call(query, schema=validate.Schema(
-            {"data": {"clip": {
-                "playbackAccessToken": {
-                    "signature": str,
-                    "value": str,
-                },
-                "videoQualities": [validate.all(
-                    {
-                        "frameRate": validate.transform(int),
-                        "quality": str,
-                        "sourceURL": validate.url(),
+        return self.call(
+            query,
+            schema=validate.all(
+                {
+                    "data": {
+                        "clip": {
+                            "playbackAccessToken": {
+                                "signature": str,
+                                "value": str,
+                            },
+                            "videoQualities": [
+                                validate.all(
+                                    {
+                                        "frameRate": validate.transform(int),
+                                        "quality": str,
+                                        "sourceURL": validate.url(),
+                                    },
+                                    validate.transform(
+                                        lambda q: (
+                                            f"{q['quality']}p{q['frameRate']}",
+                                            q["sourceURL"],
+                                        ),
+                                    ),
+                                ),
+                            ],
+                        },
                     },
-                    validate.transform(lambda q: (
-                        f"{q['quality']}p{q['frameRate']}",
-                        q["sourceURL"],
-                    )),
-                )],
-            }}},
-            validate.get(("data", "clip")),
-            validate.union_get(
-                ("playbackAccessToken", "signature"),
-                ("playbackAccessToken", "value"),
-                "videoQualities",
+                },
+                validate.get(("data", "clip")),
+                validate.union_get(
+                    ("playbackAccessToken", "signature"),
+                    ("playbackAccessToken", "value"),
+                    "videoQualities",
+                ),
             ),
-        ))
+        )
 
 
 class TwitchClientIntegrity:
@@ -591,7 +645,7 @@ class TwitchClientIntegrity:
         channel: str,
         headers: Mapping[str, str],
         device_id: str,
-    ) -> Optional[Tuple[str, int]]:
+    ) -> tuple[str, int] | None:
         from streamlink.compat import BaseExceptionGroup  # noqa: PLC0415
         from streamlink.webbrowser.cdp import CDPClient, CDPClientSession, devtools  # noqa: PLC0415
 
@@ -599,10 +653,10 @@ class TwitchClientIntegrity:
         js_get_integrity_token = cls.JS_INTEGRITY_TOKEN \
             .replace("SCRIPT_SOURCE", cls.URL_P_SCRIPT) \
             .replace("HEADERS", json_dumps(headers)) \
-            .replace("DEVICE_ID", device_id)
+            .replace("DEVICE_ID", device_id)  # fmt: skip
         eval_timeout = session.get_option("webbrowser-timeout")
         # noinspection PyUnusedLocal
-        client_integrity: Optional[str] = None
+        client_integrity: str | None = None
 
         async def on_main(client_session: CDPClientSession, request: devtools.fetch.RequestPaused):
             async with client_session.alter_request(request) as cm:
@@ -771,6 +825,7 @@ class Twitch(Plugin):
                     self._checked_metadata = True
                     self._get_metadata()
                 return parent_method()
+
             return inner
 
         parent = super()
@@ -792,7 +847,7 @@ class Twitch(Plugin):
         except (PluginError, TypeError):
             pass
 
-    def _client_integrity_token(self, channel: str) -> Optional[Tuple[str, str]]:
+    def _client_integrity_token(self, channel: str) -> tuple[str, str] | None:
         if self.options.get("purge-client-integrity"):
             log.info("Removing cached client-integrity token...")
             self.cache.set(self._CACHE_KEY_CLIENT_INTEGRITY, None, 0)
@@ -823,22 +878,23 @@ class Twitch(Plugin):
         response = ""
         data = (None, None)
 
-        # try without a client-integrity token first (the web player did the same on 2023-05-31)
-        if not self.options.get("force-client-integrity"):
+        # if live, try without a client-integrity token first (the web player did the same on 2023-05-31)
+        # if not live, we don't need a client-integrity token
+        if not is_live or not self.options.get("force-client-integrity"):
             response, *data = self.api.access_token(is_live, channel_or_vod)
 
-        # try again with a client-integrity token if the API response was erroneous
-        if response != "token":
-            client_integrity = self._client_integrity_token(channel_or_vod) if is_live else None
+        # if live and the previous API response was erroneous, try again with a client-integrity token
+        if is_live and response != "token":
+            client_integrity = self._client_integrity_token(channel_or_vod)
             response, *data = self.api.access_token(is_live, channel_or_vod, client_integrity)
 
-            # unknown API response error: abort
-            if response != "token":
-                error, message = data
-                raise PluginError(f"{error or 'Error'}: {message or 'Unknown error'}")
+        # unknown API response error: abort
+        if response != "token":
+            error, message = data
+            raise PluginError(f"{error or 'Error'}: {message or 'Unknown error'}")
 
         # access token response was empty: stream is offline or channel doesn't exist
-        if response == "token" and data[0] is None:
+        elif data[0] is None:
             raise NoStreamsError
 
         sig, token = data
@@ -891,10 +947,12 @@ class Twitch(Plugin):
                 with suppress(PluginError):
                     error = validate.Schema(
                         validate.parse_json(),
-                        [{
-                            "type": "error",
-                            "error": str,
-                        }],
+                        [
+                            {
+                                "type": "error",
+                                "error": str,
+                            },
+                        ],
                         validate.get((0, "error")),
                     ).validate(orig.response.text)
                     # Only log error messages if the channel is actually live
